package java.util;

import java.util.function.UnaryOperator;
import java.util.function.Predicate;
import java.util.function.Consumer;
import java.io.ObjectOutputStream;
import java.io.IOException;
import java.io.StreamCorruptedException;
import java.io.ObjectInputStream;
import java.io.Serializable;

public class Vector<E> extends AbstractList<E> implements List<E>, RandomAccess, Cloneable, Serializable
{
    protected Object[] elementData;
    protected int elementCount;
    protected int capacityIncrement;
    private static final long serialVersionUID = -2767605614048989439L;
    private static final int MAX_ARRAY_SIZE = 2147483639;
    
    public Vector(int i, int capacityIncrement) {
        if (i < 0) {
            throw new IllegalArgumentException("Illegal Capacity: " + i);
        }
        this.elementData = new Object[i];
        this.capacityIncrement = capacityIncrement;
        return;
    }
    
    public Vector(int n) {
        this(n, 0);
    }
    
    public Vector() {
        this(10);
    }
    
    public Vector(Collection<? extends E> collection) {
        Object[] array = collection.toArray();
        this.elementCount = array.length;
        if (collection.getClass() == ArrayList.class) {
            this.elementData = array;
        }
        else {
            this.elementData = Arrays.copyOf(array, this.elementCount, Object[].class);
        }
    }
    
    public synchronized void copyInto(Object[] array) {
        System.arraycopy(this.elementData, 0, array, 0, this.elementCount);
    }
    
    public synchronized void trimToSize() {
        ++this.modCount;
        if (this.elementCount < this.elementData.length) {
            this.elementData = Arrays.copyOf(this.elementData, this.elementCount);
        }
    }
    
    public synchronized void ensureCapacity(int n) {
        if (n > 0) {
            ++this.modCount;
            this.ensureCapacityHelper(n);
        }
    }
    
    private void ensureCapacityHelper(int n) {
        if (n - this.elementData.length > 0) {
            this.grow(n);
        }
    }
    
    private void grow(int n) {
        int length = this.elementData.length;
        int hugeCapacity = length + ((this.capacityIncrement > 0) ? this.capacityIncrement : length);
        if (hugeCapacity - n < 0) {
            hugeCapacity = n;
        }
        if (hugeCapacity - 2147483639 > 0) {
            hugeCapacity = hugeCapacity(n);
        }
        this.elementData = Arrays.copyOf(this.elementData, hugeCapacity);
    }
    
    private static int hugeCapacity(int n) {
        if (n < 0) {
            throw new OutOfMemoryError();
        }
        return (n > 2147483639) ? Integer.MAX_VALUE : 2147483639;
    }
    
    public synchronized void setSize(int i) {
        ++this.modCount;
        if (i > this.elementCount) {
            this.ensureCapacityHelper(i);
        }
        else {
            for (; i < this.elementCount; ++i) {
                this.elementData[i] = null;
            }
        }
        this.elementCount = i;
    }
    
    public synchronized int capacity() {
        return this.elementData.length;
    }
    
    @Override
    public synchronized int size() {
        return this.elementCount;
    }
    
    @Override
    public synchronized boolean isEmpty() {
        return this.elementCount == 0;
    }
    
    public Enumeration<E> elements() {
        return new Enumeration<E>() {
            int count = 0;
            
            @Override
            public boolean hasMoreElements() {
                return this.count < Vector.this.elementCount;
            }
            
            @Override
            public E nextElement() {
                synchronized (Vector.this) {
                    if (this.count < Vector.this.elementCount) {
                        return Vector.this.elementData(this.count++);
                    }
                }
                throw new NoSuchElementException("Vector Enumeration");
            }
        };
    }
    
    @Override
    public boolean contains(Object o) {
        return this.indexOf(o, 0) >= 0;
    }
    
    @Override
    public int indexOf(Object o) {
        return this.indexOf(o, 0);
    }
    
    public synchronized int indexOf(Object o, int i) {
        if (o == null) {
            while (i < this.elementCount) {
                if (this.elementData[i] == null) {
                    return i;
                }
                ++i;
            }
        }
        else {
            while (i < this.elementCount) {
                if (o.equals(this.elementData[i])) {
                    return i;
                }
                ++i;
            }
        }
        return -1;
    }
    
    @Override
    public synchronized int lastIndexOf(Object o) {
        return this.lastIndexOf(o, this.elementCount - 1);
    }
    
    public synchronized int lastIndexOf(Object o, int i) {
        if (i >= this.elementCount) {
            throw new IndexOutOfBoundsException(i + " >= " + this.elementCount);
        }
        if (o == null) {
            while (i >= 0) {
                if (this.elementData[i] == null) {
                    return i;
                }
                --i;
            }
        }
        else {
            while (i >= 0) {
                if (o.equals(this.elementData[i])) {
                    return i;
                }
                --i;
            }
        }
        return -1;
    }
    
    public synchronized E elementAt(int i) {
        if (i >= this.elementCount) {
            throw new ArrayIndexOutOfBoundsException(i + " >= " + this.elementCount);
        }
        return this.elementData(i);
    }
    
    public synchronized E firstElement() {
        if (this.elementCount == 0) {
            throw new NoSuchElementException();
        }
        return this.elementData(0);
    }
    
    public synchronized E lastElement() {
        if (this.elementCount == 0) {
            throw new NoSuchElementException();
        }
        return this.elementData(this.elementCount - 1);
    }
    
    public synchronized void setElementAt(E e, int i) {
        if (i >= this.elementCount) {
            throw new ArrayIndexOutOfBoundsException(i + " >= " + this.elementCount);
        }
        this.elementData[i] = e;
        return;
    }
    
    public synchronized void removeElementAt(int n) {
        ++this.modCount;
        if (n >= this.elementCount) {
            throw new ArrayIndexOutOfBoundsException(n + " >= " + this.elementCount);
        }
        if (n < 0) {
            throw new ArrayIndexOutOfBoundsException(n);
        }
        int n2 = this.elementCount - n - 1;
        if (n2 > 0) {
            System.arraycopy(this.elementData, n + 1, this.elementData, n, n2);
        }
        --this.elementCount;
        this.elementData[this.elementCount] = null;
        return;
    }
    
    public synchronized void insertElementAt(E e, int i) {
        ++this.modCount;
        if (i > this.elementCount) {
            throw new ArrayIndexOutOfBoundsException(i + " > " + this.elementCount);
        }
        this.ensureCapacityHelper(this.elementCount + 1);
        System.arraycopy(this.elementData, i, this.elementData, i + 1, this.elementCount - i);
        this.elementData[i] = e;
        ++this.elementCount;
        return;
    }
    
    public synchronized void addElement(E e) {
        ++this.modCount;
        this.ensureCapacityHelper(this.elementCount + 1);
        this.elementData[this.elementCount++] = e;
    }
    
    public synchronized boolean removeElement(Object o) {
        ++this.modCount;
        int index = this.indexOf(o);
        if (index >= 0) {
            this.removeElementAt(index);
            return true;
        }
        return false;
    }
    
    public synchronized void removeAllElements() {
        ++this.modCount;
        for (int i = 0; i < this.elementCount; ++i) {
            this.elementData[i] = null;
        }
        this.elementCount = 0;
    }
    
    public synchronized Object clone() {
        try {
            Vector<Object> vector = (Vector)super.clone();
            vector.elementData = Arrays.copyOf(this.elementData, this.elementCount);
            vector.modCount = 0;
            return vector;
        }
        catch (CloneNotSupportedException cause) {
            throw new InternalError(cause);
        }
    }
    
    @Override
    public synchronized Object[] toArray() {
        return Arrays.copyOf(this.elementData, this.elementCount);
    }
    
    @Override
    public synchronized <T> T[] toArray(T[] array) {
        if (array.length < this.elementCount) {
            return (T[])Arrays.copyOf(this.elementData, this.elementCount, array.getClass());
        }
        System.arraycopy(this.elementData, 0, array, 0, this.elementCount);
        if (array.length > this.elementCount) {
            array[this.elementCount] = null;
            return array;
        }
        return array;
    }
    
    E elementData(int n) {
        return this.elementData[n];
    }
    
    @Override
    public synchronized E get(int index) {
        if (index >= this.elementCount) {
            throw new ArrayIndexOutOfBoundsException(index);
        }
        return this.elementData(index);
    }
    
    @Override
    public synchronized E set(int index, E e) {
        if (index >= this.elementCount) {
            throw new ArrayIndexOutOfBoundsException(index);
        }
        Object elementData = this.elementData(index);
        this.elementData[index] = e;
        return elementData;
    }
    
    @Override
    public synchronized boolean add(E e) {
        ++this.modCount;
        this.ensureCapacityHelper(this.elementCount + 1);
        this.elementData[this.elementCount++] = e;
        return true;
    }
    
    @Override
    public boolean remove(Object o) {
        return this.removeElement(o);
    }
    
    @Override
    public void add(int n, E e) {
        this.insertElementAt(e, n);
    }
    
    @Override
    public synchronized E remove(int index) {
        ++this.modCount;
        if (index >= this.elementCount) {
            throw new ArrayIndexOutOfBoundsException(index);
        }
        Object elementData = this.elementData(index);
        int n = this.elementCount - index - 1;
        if (n > 0) {
            System.arraycopy(this.elementData, index + 1, this.elementData, index, n);
        }
        this.elementData[--this.elementCount] = null;
        return elementData;
    }
    
    @Override
    public void clear() {
        this.removeAllElements();
    }
    
    @Override
    public synchronized boolean containsAll(Collection<?> collection) {
        return super.containsAll(collection);
    }
    
    @Override
    public synchronized boolean addAll(Collection<? extends E> collection) {
        ++this.modCount;
        Object[] array = collection.toArray();
        int length = array.length;
        this.ensureCapacityHelper(this.elementCount + length);
        System.arraycopy(array, 0, this.elementData, this.elementCount, length);
        this.elementCount += length;
        return length != 0;
    }
    
    @Override
    public synchronized boolean removeAll(Collection<?> collection) {
        return super.removeAll(collection);
    }
    
    @Override
    public synchronized boolean retainAll(Collection<?> collection) {
        return super.retainAll(collection);
    }
    
    @Override
    public synchronized boolean addAll(int index, Collection<? extends E> collection) {
        ++this.modCount;
        if (index < 0 || index > this.elementCount) {
            throw new ArrayIndexOutOfBoundsException(index);
        }
        Object[] array = collection.toArray();
        int length = array.length;
        this.ensureCapacityHelper(this.elementCount + length);
        int n = this.elementCount - index;
        if (n > 0) {
            System.arraycopy(this.elementData, index, this.elementData, index + length, n);
        }
        System.arraycopy(array, 0, this.elementData, index, length);
        this.elementCount += length;
        return length != 0;
    }
    
    @Override
    public synchronized boolean equals(Object o) {
        return super.equals(o);
    }
    
    @Override
    public synchronized int hashCode() {
        return super.hashCode();
    }
    
    @Override
    public synchronized String toString() {
        return super.toString();
    }
    
    @Override
    public synchronized List<E> subList(int n, int n2) {
        return Collections.synchronizedList(super.subList(n, n2), this);
    }
    
    @Override
    protected synchronized void removeRange(int n, int n2) {
        ++this.modCount;
        System.arraycopy(this.elementData, n2, this.elementData, n, this.elementCount - n2);
        while (this.elementCount != this.elementCount - (n2 - n)) {
            this.elementData[--this.elementCount] = null;
        }
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        ObjectInputStream.GetField fields = objectInputStream.readFields();
        int value = fields.get("elementCount", 0);
        Object[] array = (Object[])(Object[])fields.get("elementData", null);
        if (value < 0 || array == null || value > array.length) {
            throw new StreamCorruptedException("Inconsistent vector internals");
        }
        this.elementCount = value;
        this.elementData = (Object[])array.clone();
        return;
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        ObjectOutputStream.PutField putFields = objectOutputStream.putFields();
        Object[] array;
        synchronized (this) {
            putFields.put("capacityIncrement", this.capacityIncrement);
            putFields.put("elementCount", this.elementCount);
            array = (Object[])this.elementData.clone();
        }
        putFields.put("elementData", array);
        objectOutputStream.writeFields();
    }
    
    @Override
    public synchronized ListIterator<E> listIterator(int i) {
        if (i < 0 || i > this.elementCount) {
            throw new IndexOutOfBoundsException("Index: " + i);
        }
        return new Vector.ListItr(i);
    }
    
    @Override
    public synchronized ListIterator<E> listIterator() {
        return new Vector.ListItr(0);
    }
    
    @Override
    public synchronized Iterator<E> iterator() {
        return new Vector.Itr();
    }
    
    @Override
    public synchronized void forEach(Consumer<? super E> consumer) {
        Objects.requireNonNull(consumer);
        int modCount = this.modCount;
        Object[] array = (Object[])this.elementData;
        for (int elementCount = this.elementCount, n = 0; this.modCount == modCount && n < elementCount; ++n) {
            consumer.accept(array[n]);
        }
        if (this.modCount != modCount) {
            throw new ConcurrentModificationException();
        }
        return;
    }
    
    @Override
    public synchronized boolean removeIf(Predicate<? super E> predicate) {
        Objects.requireNonNull(predicate);
        int n = 0;
        int elementCount = this.elementCount;
        BitSet set = new BitSet(elementCount);
        int modCount = this.modCount;
        for (int n2 = 0; this.modCount == modCount && n2 < elementCount; ++n2) {
            if (predicate.test(this.elementData[n2])) {
                set.set(n2);
                ++n;
            }
        }
        if (this.modCount != modCount) {
            throw new ConcurrentModificationException();
        }
        int n3 = n > 0;
        if (!(n3 == 0)) {
            int i = elementCount - n;
            for (int nextClearBit = 0, n4 = 0; nextClearBit < elementCount && n4 < i; nextClearBit = set.nextClearBit(nextClearBit), this.elementData[n4] = this.elementData[nextClearBit], ++nextClearBit, ++n4) {}
            for (; i < elementCount; ++i) {
                this.elementData[i] = null;
            }
            this.elementCount = i;
            if (this.modCount != modCount) {
                throw new ConcurrentModificationException();
            }
            ++this.modCount;
        }
        return n3;
    }
    
    @Override
    public synchronized void replaceAll(UnaryOperator<E> unaryOperator) {
        Objects.requireNonNull(unaryOperator);
        int modCount = this.modCount;
        for (int elementCount = this.elementCount, n = 0; this.modCount == modCount && n < elementCount; ++n) {
            this.elementData[n] = unaryOperator.apply(this.elementData[n]);
        }
        if (this.modCount != modCount) {
            throw new ConcurrentModificationException();
        }
        ++this.modCount;
        return;
    }
    
    @Override
    public synchronized void sort(Comparator<? super E> comparator) {
        int modCount = this.modCount;
        Arrays.sort((Object[])this.elementData, 0, this.elementCount, comparator);
        if (this.modCount != modCount) {
            throw new ConcurrentModificationException();
        }
        ++this.modCount;
        return;
    }
    
    @Override
    public Spliterator<E> spliterator() {
        return new Vector.VectorSpliterator(this, null, 0, -1, 0);
    }
    
    private class Itr implements Iterator<E>
    {
        int cursor;
        int lastRet;
        int expectedModCount;
        
        private Itr() {
            this.lastRet = -1;
            this.expectedModCount = Vector.this.modCount;
        }
        
        @Override
        public boolean hasNext() {
            return this.cursor != Vector.this.elementCount;
        }
        
        @Override
        public E next() {
            synchronized (Vector.this) {
                this.checkForComodification();
                int cursor = this.cursor;
                if (cursor >= Vector.this.elementCount) {
                    throw new NoSuchElementException();
                }
                this.cursor = cursor + 1;
                Vector<Object> this$0 = Vector.this;
                int lastRet = cursor;
                this.lastRet = lastRet;
                return this$0.elementData(lastRet);
            }
        }
        
        @Override
        public void remove() {
            if (this.lastRet == -1) {
                throw new IllegalStateException();
            }
            synchronized (Vector.this) {
                this.checkForComodification();
                Vector.this.remove(this.lastRet);
                this.expectedModCount = Vector.this.modCount;
            }
            this.cursor = this.lastRet;
            this.lastRet = -1;
            return;
        }
        
        @Override
        public void forEachRemaining(Consumer<? super E> consumer) {
            Objects.requireNonNull(consumer);
            synchronized (Vector.this) {
                int elementCount = Vector.this.elementCount;
                int cursor = this.cursor;
                if (cursor >= elementCount) {
                    return;
                }
                Object[] array = (Object[])Vector.this.elementData;
                if (cursor >= array.length) {
                    throw new ConcurrentModificationException();
                }
                while (cursor != elementCount && Vector.this.modCount == this.expectedModCount) {
                    consumer.accept(array[cursor++]);
                }
                this.cursor = cursor;
                this.lastRet = cursor - 1;
                this.checkForComodification();
            }
        }
        
        final void checkForComodification() {
            if (Vector.this.modCount != this.expectedModCount) {
                throw new ConcurrentModificationException();
            }
            return;
        }
    }
    
    final class ListItr extends Vector.Itr implements ListIterator<E>
    {
        ListItr(int cursor) {
            this.cursor = cursor;
        }
        
        @Override
        public boolean hasPrevious() {
            return this.cursor != 0;
        }
        
        @Override
        public int nextIndex() {
            return this.cursor;
        }
        
        @Override
        public int previousIndex() {
            return this.cursor - 1;
        }
        
        @Override
        public E previous() {
            synchronized (Vector.this) {
                this.checkForComodification();
                int cursor = this.cursor - 1;
                if (cursor < 0) {
                    throw new NoSuchElementException();
                }
                this.cursor = cursor;
                Vector<Object> this$0 = Vector.this;
                int lastRet = cursor;
                this.lastRet = lastRet;
                return this$0.elementData(lastRet);
            }
        }
        
        @Override
        public void set(E e) {
            if (this.lastRet == -1) {
                throw new IllegalStateException();
            }
            synchronized (Vector.this) {
                this.checkForComodification();
                Vector.this.set(this.lastRet, e);
            }
            return;
        }
        
        @Override
        public void add(E e) {
            int cursor = this.cursor;
            synchronized (Vector.this) {
                this.checkForComodification();
                Vector.this.add(cursor, e);
                this.expectedModCount = Vector.this.modCount;
            }
            this.cursor = cursor + 1;
            this.lastRet = -1;
        }
    }
    
    static final class VectorSpliterator<E> implements Spliterator<E>
    {
        private final Vector<E> list;
        private Object[] array;
        private int index;
        private int fence;
        private int expectedModCount;
        
        VectorSpliterator(Vector<E> list, Object[] array, int index, int fence, int expectedModCount) {
            this.list = list;
            this.array = array;
            this.index = index;
            this.fence = fence;
            this.expectedModCount = expectedModCount;
        }
        
        private int getFence() {
            int fence;
            if ((fence = this.fence) < 0) {
                synchronized (this.list) {
                    this.array = this.list.elementData;
                    this.expectedModCount = this.list.modCount;
                    int elementCount = this.list.elementCount;
                    this.fence = elementCount;
                    fence = elementCount;
                }
            }
            return fence;
        }
        
        @Override
        public Spliterator<E> trySplit() {
            int fence = this.getFence();
            int index = this.index;
            int index2 = index + fence >>> 1;
            return (index >= index2) ? null : new Vector.VectorSpliterator(this.list, this.array, index, this.index = index2, this.expectedModCount);
        }
        
        @Override
        public boolean tryAdvance(Consumer<? super E> consumer) {
            if (consumer == null) {
                throw new NullPointerException();
            }
            int index;
            if (this.getFence() > (index = this.index)) {
                this.index = index + 1;
                consumer.accept(this.array[index]);
                if (this.list.modCount != this.expectedModCount) {
                    throw new ConcurrentModificationException();
                }
                return true;
            }
            else {
                return false;
            }
        }
        
        @Override
        public void forEachRemaining(Consumer<? super E> consumer) {
            if (consumer == null) {
                throw new NullPointerException();
            }
            Vector<Object> list;
            if ((list = this.list) != null) {
                int fence;
                Object[] array;
                if ((fence = this.fence) < 0) {
                    synchronized (list) {
                        this.expectedModCount = list.modCount;
                        Object[] elementData = list.elementData;
                        this.array = elementData;
                        array = elementData;
                        int elementCount = list.elementCount;
                        this.fence = elementCount;
                        fence = elementCount;
                    }
                }
                else {
                    array = this.array;
                }
                int i;
                if (array != null && (i = this.index) >= 0 && (this.index = fence) <= array.length) {
                    while (i < fence) {
                        consumer.accept(array[i++]);
                    }
                    if (list.modCount == this.expectedModCount) {
                        return;
                    }
                }
            }
            throw new ConcurrentModificationException();
        }
        
        @Override
        public long estimateSize() {
            return (long)(this.getFence() - this.index);
        }
        
        @Override
        public int characteristics() {
            return 16464;
        }
    }
}
